COMMENT
afexp2.mod

Note:  Any mechanism that needs the values generated by Fexp2 
should be initialized AFTER Fexp2 is initialized.  This will 
happen automatically if the NMODL file that defines the dependent
mechanism starts with a letter that sorts after "a", the first 
letter in the name of this NMODL file.  Alternatively, one could
achieve the same effect by explicitly naming, in the desired sequence,
all of the NMODL files that are to be compiled by nrnivmodl.


Fexp2 is patterned after Exp2Syn, but does not generate a current or 
conductance itself.  It uses a two state kinetic scheme to generate 
a function that is the difference between two decaying exponents, 
one with time constant tau1 and the other with time constant tau2;
tau2 MUST be > tau1.
The normalized peak amplitude is 1.

This mechanism is driven by events.  Arrival of a single event with 
weight w will generate an output waveform with peak amplitude w.
The output variable is POINTER x.  

This mechanism does NOT "saturate" i.e. temporal summation 
can drive x > 1, and event weights > 1 WILL drive x > 1.


This mechanism was created in response to the need for a "distributed
synaptic conductance" (a density mechanism inserted into one or more 
sections) that could be driven by events.  Since the NMODL code for 
a density mechanism cannot (yet) contain a NET_RECEIVE block, the 
workaround I chose was to separate the event-receiving component 
from the density mechanism part, and put it in a point process.
A NetStim or other event source can then drive this mechanism via 
a NetCon, and this mechanism's POINTER x can then be used to drive
a GLOBAL variable that belongs to a density mechanism.

Response to input events:

The solution of A->G->bath with rate constants 1/tau1 and 1/tau2 is
 A = a*exp(-t/tau1) and
 G = a*tau2/(tau2-tau1)*(-exp(-t/tau1) + exp(-t/tau2))
  where tau1 < tau2

If tau2-tau1 -> 0 then we have an alpha synapse.
and if tau1 -> 0 then we have just single exponential decay.

The factor is evaluated in the initial block such that an event of 
weight 1 generates a peak response of 1.

Because the solution is a sum of exponentials, the coupled equations 
can be solved as a pair of independent equations by the cnexp method,
which is more efficient than the sparse method that would have to be
used if this were actually implemented with a kinetic scheme.
ENDCOMMENT

NEURON {
  POINT_PROCESS Fexp2
  RANGE tau1, tau2
  POINTER x
}

UNITS {
  (nA) = (nanoamp)
  (mV) = (millivolt)
  (uS) = (microsiemens)
}

PARAMETER {
  tau1 = 0.1 (ms) <1e-9,1e9>
  tau2 = 10 (ms) <1e-9,1e9>
}

ASSIGNED {
  factor (1)
  x (1)
}

STATE {
  A (1)
  B (1)
}

INITIAL {
  LOCAL tp
: printf("initializing Fexp2\n")
  if (tau1/tau2 > 0.9999) {
    tau1 = 0.9999*tau2
  }
  A = 0
  B = 0
  : note initializaton of x, which prevents each run from affecting the next
  x = B - A
  tp = (tau1*tau2)/(tau2 - tau1) * log(tau2/tau1)
  factor = -exp(-tp/tau1) + exp(-tp/tau2)
  factor = 1/factor
}

BREAKPOINT {
: printf("Fexp2\n")
  SOLVE state METHOD cnexp
: with adaptive integration, statements in the BREAKPOINT block
: are not executed unless a current is being written to
: that's why the following statement must be placed 
: in the DERIVATIVE block
:  x = B - A
}

DERIVATIVE state {
  A' = -A/tau1
  B' = -B/tau2
  x = B - A
}

NET_RECEIVE(weight (1)) {
  A = A + weight*factor
  B = B + weight*factor
}
